/*-
 * Copyright (c) 2001 Lev Walkin <vlm@spelio.net.ru>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: ppd.c,v 1.19 2002/12/25 10:09:48 vlm Exp $
 */

#include "ipcad.h"
#include "rw.h"
#include "opt.h"
#include "storage.h"

struct ip *
find_ipv4_level(packet_source_t *ps, char *packet, int caplen) {
	register struct ip *ipv4h = NULL;


	switch(ps->dlt) {
	case DLT_EN10MB:

		ipv4h = (struct ip *)(packet + ETHER_HDR_LEN);

		break;
	case DLT_PPP:
#ifdef	DLT_C_HDLC
	case DLT_C_HDLC:
#endif
		ipv4h = (struct ip *)(packet + 4);
		break;
	case DLT_RAW:	/* Some PPP implementations */
		/*
		 * Packets are always IP.
		 */
		ipv4h = (struct ip *)packet;
		break;
	case DLT_NULL:
		ipv4h = (struct ip *)(packet + 4);
		break;
#ifdef	DLT_LINUX_SLL
	case DLT_LINUX_SLL:	/* Linux cooked socket */
		ipv4h = (struct ip *)(packet + 16);
		break;
#endif
	case DLT_IEEE802:	/* Token Ring */
		ipv4h = (struct ip *)(packet + 22);
		break;
	default:
		return NULL;
	}

	if(caplen - (int)((char *)ipv4h - (char *)packet)
		< sizeof(struct ip)) {
		/* It is not complete IP packet */
		return NULL;
	}

	if(ipv4h->ip_v != IPVERSION)
		return NULL;

	return ipv4h;
}

void
process_packet_data(
	unsigned char *psp,
#ifdef	PSRC_bpf
	const struct bpf_hdr *bh,
#else
	const struct pcap_pkthdr *ph,
#endif	/* PSRC_bpf */
	const unsigned char *packet
) {
#ifdef	PSRC_bpf
	int caplen = bh->bh_caplen;
#else
	int caplen = ph->caplen;
#endif	/* PSRC_bpf */

	flow_t flow;
	struct ip *ipv4h;
	int iplen;

	packet_source_t *psrc = ((packet_source_t *)psp);

	ipv4h = find_ipv4_level(psrc, (char *)packet, caplen);
	if(!ipv4h)
		return;

	iplen = ntohs(ipv4h->ip_len);

	/*
	 * Enter critical section
	 */

#ifdef	HAVE_PTHREAD_CANCEL
	pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
#endif

	flow.src = ipv4h->ip_src;
	flow.dst = ipv4h->ip_dst;
	flow.packets = 1;
	flow.bytes = iplen;
	flow_update(&main_storage, &flow);

#ifdef	HAVE_PTHREAD_CANCEL
	pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
#endif

	/*
	 * Enf of critical section
	 */

	/* Perform statistics counting */
	psrc->bytes_cur += iplen;
	psrc->packets_cur += 1;

}

