/*-
 * Copyright (c) 2001 Lev Walkin <vlm@spelio.net.ru>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: pps.c,v 1.16 2002/12/25 10:09:48 vlm Exp $
 */


#include "ipcad.h"
#include "opt.h"

int
process_packet_sources(packet_source_t *pss) {
	packet_source_t *ps;
	sigset_t set, oset;

	/*
	 * Block all signals.
	 * The signal mask is inherited from the creating thread.
	 */
	sigfillset(&set);
	sigprocmask(SIG_BLOCK, &set, &oset);

	/* Run threads */
	for(ps = pss; ps; ps = ps->next) {
		if( pthread_create(&ps->thid, NULL, pps_loop, ps) ) {
			fprintf(stderr,
				"Can't start thread for %s!\n",
				ps->ifname);
			signoff_now = 1;
			sigprocmask(SIG_SETMASK, &oset, NULL);
			return -1;
		} else {
			printf("Thread %ld processing %s started.\n",
				(long)ps->thid, ps->ifname);
		}
	}

	sigprocmask(SIG_SETMASK, &oset, NULL);

	/* Signal-processing loop */

#define	SLEEP_TIME	2	/* Seconds */

	while(1) {
		sleep(SLEEP_TIME);

		if(signoff_now)
			break;

		if(display_now) {
			if(daemon_mode) {   
				FILE *f = fopen("/dev/console", "w");
				if(f) {
					display(f, 0);
					fclose(f);
				}
			} else {
				display(stdout, 0);
			}
			display_now = 0;
		}
	}

	return 0;
}

